<?php declare(strict_types=1);

namespace DoctrineMigrations;

use Doctrine\DBAL\DBALException;
use Doctrine\DBAL\Schema\Schema;
use Doctrine\Migrations\AbstractMigration;

/**
 * Class Version20200108172607
 * @package DoctrineMigrations
 */
final class Version20200108172607 extends AbstractMigration
{
    public function getDescription() : string
    {
        return 'Initial database setup';
    }

    public function up(Schema $schema) : void
    {
        try {
            $this->abortIf(
                $this->connection->getDatabasePlatform()->getName() !== 'mysql',
                'Migration can only be executed safely on \'mysql\'.'
            );
        } catch (DBALException $e) {
            $this->abortIf(true, sprintf('Migration failed! Reason: %s', $e->getMessage()));
        }
        $this->addSql('
            CREATE TABLE `mls_admin_user` (
                `id` INT AUTO_INCREMENT NOT NULL,
                `username` VARCHAR(180) NOT NULL,
                `roles` JSON NOT NULL,
                `password` VARCHAR(255) NOT NULL,
                PRIMARY KEY(`id`)
            )
            DEFAULT CHARACTER SET utf8mb4
            COLLATE `utf8mb4_unicode_ci`
            ENGINE = InnoDB
        ');
        $this->addSql('
            CREATE TABLE `mls_topic` (
                `id` INT AUTO_INCREMENT NOT NULL,
                `label` VARCHAR(20) NOT NULL,
                PRIMARY KEY(`id`)
            )
            DEFAULT CHARACTER SET utf8mb4
            COLLATE `utf8mb4_unicode_ci`
            ENGINE = InnoDB
        ');
        $this->addSql('
            CREATE TABLE `mls_question` (
                `id` INT AUTO_INCREMENT NOT NULL, 
                `topic_id` INT NOT NULL,
                `label` VARCHAR(1024) NOT NULL,
                `structure_type` VARCHAR(10) NOT NULL,
                `content_type` VARCHAR(20) NOT NULL,
                INDEX `IDX_B6F7494E1F55203D` (`topic_id`),
                CONSTRAINT `FK_B6F7494E1F55203D` FOREIGN KEY (`topic_id`) REFERENCES `mls_topic`(`id`) ON DELETE CASCADE,
                PRIMARY KEY(`id`)
            )
            DEFAULT CHARACTER SET utf8mb4
            COLLATE `utf8mb4_unicode_ci`
            ENGINE = InnoDB
        ');
        $this->addSql('
            CREATE TABLE `mls_answer` (
                `id` INT AUTO_INCREMENT NOT NULL,
                `question_id` INT NOT NULL,
                `label` VARCHAR(1024) NOT NULL,
                `is_correct` BOOL NOT NULL,
                INDEX `IDX_DADD4A251E27F6BF` (`question_id`),
                CONSTRAINT `FK_DADD4A251E27F6BF` FOREIGN KEY (`question_id`) REFERENCES `mls_question`(`id`) ON DELETE CASCADE,
                PRIMARY KEY(`id`)
            )
            DEFAULT CHARACTER SET utf8mb4
            COLLATE `utf8mb4_unicode_ci`
            ENGINE = InnoDB
        ');
    }

    public function down(Schema $schema) : void
    {
        try {
            $this->abortIf(
                $this->connection->getDatabasePlatform()->getName() !== 'mysql',
                'Migration can only be executed safely on \'mysql\'.'
            );
        } catch (DBALException $e) {
            $this->abortIf(true, sprintf('Migration failed! Reason: %s', $e->getMessage()));
        }

        $this->addSql('ALTER TABLE `mls_answer` DROP FOREIGN KEY `FK_DADD4A251E27F6BF`');
        $this->addSql('ALTER TABLE `mls_question` DROP FOREIGN KEY `FK_B6F7494E1F55203D`');
        $this->addSql('DROP TABLE `mls_answer`');
        $this->addSql('DROP TABLE `mls_question`');
        $this->addSql('DROP TABLE `mls_topic`');
        $this->addSql('DROP TABLE `mls_admin_user`');
    }
}
